package LifeGame;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;
import java.util.Random;
import java.lang.*;

public class LifePool {
    public static final boolean ALIVE = true;
    public static final boolean DEATH = false;

    public static final int EAST = 0;
    public static final int SOUTH = 1;
    public static final int WEST = 2;
    public static final int NORTH = 3;
    public static final int CENTER = 4;

    final int diroffset[][] = {
        {0, 1}, {1, 0}, {0, -1}, {-1, 0} 
    };

    private int edgeLength;
    private boolean[][][] cellPool;
    private int whichOne; 

    private int stageCount;

    // Constrcutor
    public LifePool(int el) {   // make a square
        edgeLength = el;

        cellPool = new boolean[2][][];
        cellPool[0] = new boolean[edgeLength][edgeLength];
        cellPool[1] = new boolean[edgeLength][edgeLength];

        whichOne = 0;
        stageCount = 0;

        purify();
    }

    // raze all life
    public void purify() {
        int i, j;

        for (i=0; i<edgeLength; i++) {        
            for (j=0; j<edgeLength; j++) {
                cellPool[0][i][j] = cellPool[1][i][j] = DEATH;
            }
        }

        stageCount = 0;
    }

    public String toString() {
        String tmp = "Stage: ";
        int i, j;

        tmp = tmp + stageCount + ":" + whichOne + "\n";
        for (i=0; i<edgeLength; i++) {
            for (j=0; j<edgeLength; j++) {
                if (cellPool[whichOne][i][j] == ALIVE) {
                    tmp = tmp + "1";
                } else {
                    tmp = tmp + "0";
                }
            }
            tmp = tmp + "\n";
        }
        return tmp;
    }

    public int getEdgeLength() {
        return edgeLength;
    }

    public int getStageCount() {
        return stageCount;
    }

    private boolean getAroundCell(int x, int y, int dir) {
        int tmpx, tmpy;

        tmpx = x + diroffset[dir][0];
        tmpy = y + diroffset[dir][1];

        if (tmpx < 0 || tmpx >= edgeLength ||
            tmpy < 0 || tmpy >= edgeLength) return DEATH;

        return cellPool[whichOne][tmpx][tmpy];    
    }

    public int setCell(int x, int y, boolean doa) {
        if (x<0 || x>=edgeLength ||
            y<0 || y>=edgeLength) return -1;
        cellPool[whichOne][x][y] = doa;
        return 0;
    }

    public boolean getCell(int x, int y) {
        if (x<0 || x>=edgeLength ||
            y<0 || y>=edgeLength) return DEATH;
        return cellPool[whichOne][x][y];
    }
     
    public void setPool(boolean[] poolData) {
        int p, i, j;
        boolean doa;

        for (p=0,i=0; i<edgeLength; i++) {
            for (j=0; j<edgeLength; j++) {
                if (p<poolData.length) {
                    doa = poolData[p];
                } else {
                    doa = DEATH;
                }
                cellPool[whichOne][i][j] = doa;
                p++;
            }
        }
    }

    public boolean[] getPool() {
        int i, j, p;
        boolean[] tmpPool = new boolean[edgeLength*edgeLength];

        for (p=0,i=0; i<edgeLength; i++)
            for (j=0; j<edgeLength; j++) 
                tmpPool[p++] = cellPool[whichOne][i][j];
        return tmpPool;
                
    }

    public void setChaosPool() {
        int i, j;
        Random random = new Random();
        int tmp;

        for (i=0; i<edgeLength; i++) {
            for (j=0; j<edgeLength; j++) {
                    tmp = random.nextInt() & 0x000001;
                    cellPool[whichOne][i][j] = (tmp==1)?ALIVE:DEATH;
            }
        }
    }

    public void nextStage() {
        int curPool = whichOne;
        int nextPool = (whichOne==0)?1:0;
        int i, j;

        for (i=0; i<edgeLength; i++) {
            for (j=0; j<edgeLength; j++) {
/*
                    cellPool[nextPool][i][j] = LR.getNextStatus(
                            getAroundCell(i, j, EAST),
                            getAroundCell(i, j, SOUTH),
                            getAroundCell(i, j, WEST),
                            getAroundCell(i, j, NORTH));
*/
                    cellPool[nextPool][i][j] = getNextStatus(
                            getAroundCell(i, j, EAST),
                            getAroundCell(i, j, SOUTH),
                            getAroundCell(i, j, WEST),
                            getAroundCell(i, j, NORTH));
            }
        }

        stageCount++;
        whichOne = nextPool;
    } 

    private boolean getNextStatus(boolean E, boolean S, boolean W, boolean N) {
        if (S) return ALIVE;
        else return DEATH;
    }

    public void init() {
        setCell(1, 1, ALIVE);
    }
}
